/******************************************************************************
*******************************************************************************
*******************************************************************************

    Fam++ 2
    Copyright (C) 2001 Ben Martin

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    For more details see the COPYING file in the root directory of this
    distribution.

    $Id: Fampp2GlibSupport.cpp,v 1.1 2005/07/04 11:14:06 ben Exp $

*******************************************************************************
*******************************************************************************
******************************************************************************/

#include <Fampp2GlibSupport.hh>
#include <glib.h>

#include <iostream>
using namespace std;


namespace Fampp
{
    ExceptionList_t ExceptionList;
    
    ExceptionList_t getExceptions()
    {
        return ExceptionList;
    }
    void
    clearExceptions()
    {
        ExceptionList.clear();
    }
    
    bool
    haveMoreExceptions()
    {
        return !ExceptionList.empty();
    }
    
    std::exception
    getNextException()
    {
        std::exception r = ExceptionList.front();
        ExceptionList.pop_front();
        return r;
    }
    
    
    void
    addException( const exception& e )
    {
        ExceptionList.push_back(e);
    }
    
    static gboolean
    registerWithGLib_cb( GIOChannel *source,
                         GIOCondition condition,
                         gpointer data)
    {
        while( Fampp::Instance().Pending() )
        {
            try
            {
                Fampp::Instance().NextEvent();
            }
            catch( std::exception& e )
            {
                cerr << "fampp2 exception: " << e.what() << endl;
                // Cant throw through glib callback
                // throw;
                addException( e );
            }
        }
        return 1; // call again
    }
    
    void
    ProcessFamppEvents()
    {
        while( Fampp::Instance().Pending() )
        {
            Fampp::Instance().NextEvent();
        }
    }
    
    

    void
    RegisterFamppWithGLib()
    {
        static bool alreadyRegistered = false;
        
        if( !alreadyRegistered )
        {
            alreadyRegistered = true;
        
            guint result;
            GIOChannel *channel;
            GIOCondition cond = GIOCondition(G_IO_IN | G_IO_ERR | G_IO_PRI);

//            cerr << "Attaching to fd:" << Fampp::Instance().getFD() << endl;
            
            channel = g_io_channel_unix_new( Fampp::Instance().getFD() );
            result  = g_io_add_watch( channel, cond,
                                      registerWithGLib_cb, 0 );
            g_io_channel_unref (channel);
        
        }
    }
 
};

